﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization.Encodings.Crossovers {
  /// <summary>
  /// 
  /// </summary>
  [Item("ParameterVectorCrossover", "TODO")]
  [StorableClass]
  public class ParameterConfigurationCrossover : SingleSuccessorOperator, IParameterConfigurationOperator, ICrossover {
    public override bool CanChangeName {
      get { return false; }
    }

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<ItemArray<IParameterConfiguration>> ParentsParameter {
      get { return (ScopeTreeLookupParameter<IParameterConfiguration>)Parameters["Parents"]; }
    }
    public ILookupParameter<IParameterConfiguration> ChildParameter {
      get { return (ILookupParameter<IParameterConfiguration>)Parameters["Child"]; }
    }

    private IIntegerVectorCrossover integerVectorCrossover;
    public IIntegerVectorCrossover IntegerVectorCrossover {
      get { return integerVectorCrossover; }
      set { integerVectorCrossover = value; }
    }
    private IRealVectorCrossover realVectorCrossover;
    public IRealVectorCrossover RealVectorCrossover {
      get { return realVectorCrossover; }
      set { realVectorCrossover = value; }
    }

    [StorableConstructor]
    protected ParameterConfigurationCrossover(bool deserializing) : base(deserializing) { }
    protected ParameterConfigurationCrossover(ParameterConfigurationCrossover original, Cloner cloner) : base(original, cloner) { }
    public ParameterConfigurationCrossover()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic crossover operators."));
      Parameters.Add(new ScopeTreeLookupParameter<IParameterConfiguration>("Parents", "The parent vectors which should be crossed."));
      ParentsParameter.ActualName = "ParameterConfiguration";
      Parameters.Add(new LookupParameter<IParameterConfiguration>("Child", "The child vector resulting from the crossover."));
      ChildParameter.ActualName = "ParameterConfiguration";
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ParameterConfigurationCrossover(this, cloner);
    }

    public override IOperation Apply() {

      IParameterConfiguration child = (IParameterConfiguration)ParentsParameter.ActualValue[0].Clone();
      // todo
      //for (int i = 0; i < child.Parameters.Count; i++) {
      //  if (child.Parameters[i].Parameter.DataType == typeof(IntValue)) {
      //    child.Parameters[i].Parameter.ActualValue = CrossInteger(ParentsParameter.ActualValue[0].Parameters[i], ParentsParameter.ActualValue[1].Parameters[i]);
      //  }
      //}

      this.ChildParameter.ActualValue = child;
      return base.Apply();
    }

    private IntValue CrossInteger(IParameterConfiguration parameter1, IParameterConfiguration parameter2) {
      IntegerVector integerChild = HeuristicLab.Encodings.IntegerVectorEncoding.DiscreteCrossover.Apply(
        RandomParameter.ActualValue,
        new IntegerVector(new IntArray(new int[] { ((IntValue)parameter1.ActualValueConfiguration).Value })),
        new IntegerVector(new IntArray(new int[] { ((IntValue)parameter2.ActualValueConfiguration).Value })));
      return new IntValue(integerChild[0]);
    }

  }
}
