﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class ConstrainedValue : Item {
    [Storable]
    protected Type valueDataType;
    public Type ValueDataType {
      get { return this.valueDataType; }
      protected set { this.valueDataType = value; }
    }

    [Storable]
    protected IItem value;
    public IItem Value {
      get { return value; }
      set {
        if (this.value != value) {
          if(this.value != null) DeregisterEvents();
          this.value = value;
          if(this.value != null) RegisterEvents();
          OnToStringChanged();
        }
      }
    }

    private void RegisterEvents() {
      this.value.ToStringChanged += new EventHandler(value_ToStringChanged);
    }
    private void DeregisterEvents() {
      this.value.ToStringChanged -= new EventHandler(value_ToStringChanged);
    }
    
    public ConstrainedValue() { }
    public ConstrainedValue(IItem value, Type valueDataType) {
      this.Value = value;
      this.ValueDataType = valueDataType;
    }
    protected ConstrainedValue(ConstrainedValue original, Cloner cloner) : base(original, cloner) {
      this.valueDataType = original.valueDataType;
      this.value = cloner.Clone(value);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ConstrainedValue(this, cloner);
    }

    void value_ToStringChanged(object sender, EventArgs e) {
      OnToStringChanged();
    }

    public override string ToString() {
      return value != null ? value.ToString() : base.ToString();
    }
  }
}
